#!/usr/bin/env python3
# -*- coding: utf-8 -*-
""" script for running in the terminal
@author: YangyangLi
@contact:li002252@umn.edu
@license: MIT Licence
@file: __main__.py
@time: 2020/12/21 5:02 PM
"""
from cli import cli

if __name__ == "__main__":
    cli()
